#include "dfs.hh"

extern DFS *dfs;

int POSIXBase::get_stat(string path, struct stat *attr) {
  return stat(fullpath(path).c_str(), attr);
}

void POSIXBase::read(void *buffer, string path, off_t offset, size_t size) {
  FILE *f = fopen(fullpath(path).c_str(), "rb");
  assert(f != NULL);
  assert(fseek(f, offset, SEEK_SET) == 0);
  assert(fread(buffer, size, 1, f) == 1);
  fclose(f);
}

inode_id POSIXBase::lookup(string directory, string name) {
  
  DIR *d = opendir(fullpath(directory).c_str());
  if(d == NULL) 
    return 0;

  struct dirent *ent = readdir(d);

  while(ent != NULL) {
    if(string(ent->d_name) == name) {
      string child_path = directory + string("/") + name;
      inode_id child = dfs->baptize(ent->d_ino, child_path);
      assert(child != 0);
      return child;
    }
    ent = readdir(d);
  }
  return 0;
}

void POSIXBase::ls(string directory, list<string> &entries) {
  list<ino_t> inos;
  list<string> paths;

  ls(directory, entries, inos, paths);
}

void POSIXBase::ls(string directory, 
		   list<string> &entries,
		   list<ino_t> &inos, 
		   list<string> &paths) {

  DIR *d = opendir(fullpath(directory).c_str());  
  if(d == NULL) 
    return;
  
  struct dirent *ent = readdir(d);

  while(ent != NULL) {
    string n(ent->d_name);
    if(n == string(".") || n == string("..")) {
      ent = readdir(d);
      continue;
    }
    entries.push_back(n);
    inos.push_back(ent->d_ino);
    string child_path = directory + string("/") + n;
    paths.push_back(child_path);
    ent = readdir(d);
  }
}
