#include <time.h>
#include <pthread.h>
#include <fstream>
#include <sstream>
#include <string>
#include <list>
#include <vector>
#include <algorithm>
#include <map>
#include <sfs/async.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>

#include <fcntl.h>
#include <openssl/sha.h>

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/serialization/list.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/base_object.hpp>


#ifndef BASE_INCLUDED
#define BASE_INCLUDED

#include "dfs_types.h"
#include "log.hh"

using namespace std;

class Base {
public:
  string root_path;
  Base() {}

  virtual void set_type(string path, short type) {}

  virtual int get_stat(string path, struct stat *attr) {return 0;}
  
  virtual void read(void *buffer, string path, off_t offset, size_t size) { assert(false); }
  
  virtual Event *freeze() {
    cout << "Freeze not supported by this base\n";
    return NULL;
  }

  virtual inode_id lookup(string directory, string name) { return 0; }

  virtual void ls(string directory, list<string> &entries) {}

  virtual void ls(string directory, 
		  list<string> &entries,
		  list<ino_t> &inos, 
		  list<string> &paths) {}
};


class POSIXBase : public Base {
private:
  string fullpath(string p) {
    if(p.size() == 0) 
      return root;
    else
      return root + "/" + p;
  }

public:
  string root;
  
  POSIXBase(string root) : Base() {
    this->root = root;
  }
  
  virtual int get_stat(string path, struct stat *attr);
  
  virtual void read(void *buffer, string path, off_t offset, size_t size);


  virtual inode_id lookup(string directory, string name);

  virtual void ls(string directory, list<string> &entries);

  virtual void ls(string directory, 
		  list<string> &entries,
		  list<ino_t> &inos, 
		  list<string> &paths);
  
};


#endif
