#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#include "dfs_fuse.h"

extern DFS *dfs;

/* called by fuse_lowlevel_new with userdata param */
void dfs_init(void *userdata, struct fuse_conn_info *conn) {
  dfs->read_volume();
}

/* man 2 path_resolution */
void dfs_lookup(fuse_req_t req, fuse_ino_t parent, const char *name) {
  P;
  dfs->dfs_lookup(req, parent, name);
  V;
}

/* man 2 stat ( also check http://en.wikipedia.org/wiki/Stat_(Unix) ) */
void dfs_getattr(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi) {
  P;
  dfs->dfs_getattr(req, ino, fi);
  V;
}

/* man 2 stat, man 2 chmod */
void dfs_setattr(fuse_req_t req, fuse_ino_t ino, struct stat *attr,
		 int to_set, struct fuse_file_info *fi) {
  P;
  dfs->dfs_setattr(req, ino, attr, to_set, fi);
  V;
}

/* man 2 mknod */
void dfs_mknod(fuse_req_t req, fuse_ino_t parent, const char *name,
	       mode_t mode, dev_t rdev) {
  P; 
  dfs->dfs_mknod(req, parent, name, mode, rdev);
  V;
}

/* man 2 mkdir */
void dfs_mkdir(fuse_req_t req, fuse_ino_t parent, const char *name,
	       mode_t mode) {
  P;
  dfs->dfs_mkdir(req, parent, name, mode);
  V;
}

/* man 2 unlink */
void dfs_unlink(fuse_req_t req, fuse_ino_t parent, const char *name) {
  P;
  dfs->dfs_unlink(req, parent, name);
  V;
}

/* man 2 rmdir */
void dfs_rmdir(fuse_req_t req, fuse_ino_t parent, const char *name) {
  P;
  dfs->dfs_rmdir(req, parent, name);
  V;
}

/* man 2 rename --- annoyingly intricate
 * in principle, like unlink/rmdir + mknod/mkdir modulo corner cases Fuse 
 * takes care of the fact that newparent may not be a subdirectory of parent.
 */
void dfs_rename(fuse_req_t req, fuse_ino_t parent, const char *name,
		fuse_ino_t newparent, const char *newname) {
  P;
  dfs->dfs_rename(req, parent, name, newparent, newname);
  V;
}

/* man 2 read */
void dfs_read(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off,
	      struct fuse_file_info *fi) {
  P;
  dfs->dfs_read(req, ino, size, off, fi);
  V;
}

/* man 2 write, also check ``man 2 lseek'' for out of bounds write */
void dfs_write(fuse_req_t req, fuse_ino_t ino, const char *buf,
	       size_t size, off_t off, struct fuse_file_info *fi) {
  P;
  dfs->dfs_write(req, ino, buf, size, off, fi);
  V;
}

/* man 2 readdir */
void dfs_readdir(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off,
		 struct fuse_file_info *fi) {
  P;
  dfs->dfs_readdir(req, ino, size, off, fi);
  V;
}

void dfs_destroy(void *userdata) {
  P;
  dfs->dfs_destroy(userdata);
  V;
}


void dfs_statfs(fuse_req_t req, fuse_ino_t ino) {
  P;
  dfs->dfs_statfs(req,ino);
  V;
}
