#ifndef DFS_FUSE_INCLUDED
#define DFS_FUSE_INCLUDED


#include "dfs.hh"



#ifdef __cplusplus
extern "C" {
#endif

#include <fuse_lowlevel.h>

  void dfs_statfs(fuse_req_t req, fuse_ino_t ino);

/* called by fuse_lowlevel_new with userdata param */
  void dfs_init(void *userdata, struct fuse_conn_info *conn);

/* man 2 path_resolution */
void dfs_lookup(fuse_req_t req, fuse_ino_t parent, const char *name);

/* man 2 stat ( also check http://en.wikipedia.org/wiki/Stat_(Unix) ) */
void dfs_getattr(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi);

/* man 2 stat, man 2 chmod */
void dfs_setattr(fuse_req_t req, fuse_ino_t ino, struct stat *attr,
		int to_set, struct fuse_file_info *fi);

/* man 2 mknod */
void dfs_mknod(fuse_req_t req, fuse_ino_t parent, const char *name,
		mode_t mode, dev_t rdev);

/* man 2 mkdir */
void dfs_mkdir(fuse_req_t req, fuse_ino_t parent, const char *name,
		mode_t mode);

/* man 2 unlink */
void dfs_unlink(fuse_req_t req, fuse_ino_t parent, const char *name);

/* man 2 rmdir */
void dfs_rmdir(fuse_req_t req, fuse_ino_t parent, const char *name);

/* man 2 rename --- annoyingly intricate
 * in principle, like unlink/rmdir + mknod/mkdir modulo corner cases Fuse 
 * takes care of the fact that newparent may not be a subdirectory of parent.
 */
void dfs_rename(fuse_req_t req, fuse_ino_t parent, const char *name,
		fuse_ino_t newparent, const char *newname);

/* man 2 open -- do not use if filesystem is stateless */
void dfs_open(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi);

/* man 2 close -- do not use if filesystem is stateless */
void dfs_release(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi);

/* man 2 read */
void dfs_read(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off,
		struct fuse_file_info *fi);

/* man 2 write, also check ``man 2 lseek'' for out of bounds write */
void dfs_write(fuse_req_t req, fuse_ino_t ino, const char *buf,
		       size_t size, off_t off, struct fuse_file_info *fi);

/* man 2 readdir */
void dfs_readdir(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off,
		struct fuse_file_info *fi);

void dfs_destroy(void *userdata);


static inline void dfs_ops_init(struct fuse_lowlevel_ops *ops)
{
	/* do not remove this unless all functions are plugged in the struct */
  memset(ops, 0, sizeof(struct fuse_lowlevel_ops));
  
  /* Fill in all newly implemented operations */
  ops->init = dfs_init;
  ops->lookup = dfs_lookup;
  ops->getattr = dfs_getattr;
  ops->readdir = dfs_readdir;
  ops->mkdir = dfs_mkdir;
  ops->mknod = dfs_mknod;
  ops->setattr = dfs_setattr;
  ops->write = dfs_write;
  ops->unlink = dfs_unlink;
  ops->rmdir = dfs_rmdir;
  ops->rename = dfs_rename;
  ops->read = dfs_read;
  ops->statfs = dfs_statfs;
  //  ops->open = dfs_open;
  //ops->release = dfs_release;
  ops->destroy = dfs_destroy;
}
  
#ifdef __cplusplus
}
#endif


#endif
