/*
 * @author Tudor Marian <tudorm@cs.cornell.edu>
 */

#ifndef _S3_OPS_H_
#define _S3_OPS_H_

#include "libs3.h"
#include <sys/types.h>

/* Wrapper operations over the libs3 functions, not touching bucket or meta */

#ifdef __cplusplus
extern "C"
{
#endif

/* copies the pointers (not content) to the access key and the secret key */
void s3_init_credentials(const char *access_key, const char *secret_key);

/* remove object altogether, returns true (1) on success */
int s3_rm_object(const char *bucket, const char *key);

/* 
 * buf is allocated and returned by the function, notice the (void **) type
 * @return the amount of bytes read, -1 on error. If the key contains a 0 byte
 * value, *buf will contain NULL, and the return value is 0.
 */
ssize_t s3_get_object(const char *bucket, const char *key, void **buf);

/* 
 * buf is provided by the caller, and should have at least byte_count bytes
 * @return the amount of bytes read, -1 on error.
 */ 
ssize_t s3_read_object(const char *bucket, const char *key, void *buf, 
		uint64_t start_byte, uint64_t byte_count);

/* 
 * S3 only supports full object overload, buf contains byte_count bytes 
 * @return the amount of bytes written, -1 on error.
 */
ssize_t s3_put_object(const char *bucket, const char *key, void *buf, 
		uint64_t byte_count);

/* clear all objects from the bucket, returns true (1) on success */
int s3_clear_bucket(char *bucket);

#ifdef __cplusplus
}
#endif

#endif //_S3_OPS_H_

