#include "dfs.hh"
#include "s3store.hh"

#ifndef S3BASE_INCLUDED
#define S3BASE_INCLUDED

class S3ROBase : public Base {
private:
  s3ro::S3RO *base;
  map<string,s3ro::keytype> typemap;
  
  void record_type(string key, s3ro::keytype type);
  void set_type(string key, s3ro::keytype type);

  // used only for freezing
  map<inode_id, ino_t> reverse_inode_map;
  ino_t max_ino;

public:
  string root_hash;
  string bucket;
  string keyid;
  string secret_key;

  S3ROBase(string root_hash, string bucket, string keyid, string secret_key) : Base() {
    this->root_hash = root_hash;
    this->bucket = bucket;
    this->keyid = keyid;
    this->secret_key = secret_key;

    s3ro::S3HashStore *store = new s3ro::S3HashStore(bucket, keyid, secret_key);
    s3ro::LRUStore *caching_store = new s3ro::LRUStore(store);
    root_path = root_hash;
    typemap[root_hash] = s3ro::S3RO_DIRECTORY;
    base = new s3ro::S3RO(caching_store, root_hash);
  }
  
  virtual int get_stat(string path, struct stat *attr);
  
  virtual void read(void *buffer, string path, off_t offset, size_t size);

  virtual inode_id lookup(string directory, string name);
  
  virtual void ls(string directory, list<string> &entries);
  
  virtual void ls(string directory, 
		  list<string> &entries,
		  list<ino_t> &inos, 
		  list<string> &paths);
  
  virtual Event *freeze();
  ino_t freeze_get_ino(inode_id i);
  string freeze_summary(Summary *s);
  string freeze_file_summary(FileSummary *s);
  string freeze_dir_summary(DirectorySummary *s);
};


#endif
