
#include "s3store.hh"

#include <openssl/sha.h>

using namespace s3ro;

static string base16(const unsigned char *buffer, unsigned int length) {
  char temp[length * 2];
  unsigned char c;
  for(unsigned int i = 0; i < length; i++) {
    c = buffer[i] >> 4;
    if(c < 10) temp[2*i] = '0' + c;
    else temp[2*i] = 'a'+ (c-10);
    c = buffer[i] & 0x0f;
    if(c < 10) temp[2*i+1] = '0' + c;
    else temp[2*i+1] = 'a'+ (c-10);    
  }
  return string( (const char *) temp, (size_t) (length*2));
}


static string sha1(string data) {
  SHA_CTX sha1_context;
  SHA1_Init(&sha1_context);
  SHA1_Update(&sha1_context, data.c_str(), data.length());
  unsigned char buffer[SHA_DIGEST_LENGTH];
  SHA1_Final(buffer, &sha1_context);
  //  return string( (const char *) buffer, SHA_DIGEST_LENGTH);
  return base16(buffer, SHA_DIGEST_LENGTH);
}


S3HashStore::S3HashStore(string bucket, string keyid, string secret_key) {
  this->bucket = bucket;
  this->s3_access_key_id = keyid;
  this->s3_secret_access_key = secret_key;
  
  s3_init_credentials(s3_access_key_id.c_str(), s3_secret_access_key.c_str());

}

string S3HashStore::get(string key) {
  void *buffer = NULL;
  ssize_t bytes_read = s3_get_object(bucket.c_str(), key.c_str(), &buffer);

  assert(bytes_read >= 0);

  string rval;

  if(bytes_read > 0) {
    rval = string((char*) buffer,bytes_read);
    free(buffer);
  }

  // check hash
  if(key != sha1(rval)) {
    cout << "ERROR hash does not match, object " << key << "("<< bytes_read << " bytes read)\n";
    assert(false);
  }
  return rval;
}

string S3HashStore::put(string data) {
  string key = sha1(data);
  while(1) {
    ssize_t bytes_written = s3_put_object(bucket.c_str(), key.c_str(), (void *) data.c_str(), data.size());
  
    if(bytes_written == (ssize_t) data.size()) break;
    
    cerr << "Error uploading object " << key << " to S3, retrying\n";
  }

  return key;
}

string S3HashStore::put_unless(string data, string original_key) {
  string key = sha1(data);

  if(key == original_key)
    return original_key;

  while(1) {
    ssize_t bytes_written = s3_put_object(bucket.c_str(), key.c_str(), (void *) data.c_str(), data.size());
  
    if(bytes_written == (ssize_t) data.size()) break;
    
    cerr << "Error uploading object " << key << " to S3, retrying\n";
  }

  return key;
}



void S3HashStore::ls(list<string> &l) {
  assert(false); // not yet implemented
}
