
#include "s3ro.hh"
#include "s3ops.h"

#ifndef _S3STORE_INCLUDED
#define _S3STORE_INCLUDED

namespace s3ro {

class S3HashStore : public HashStore {
private:
  string bucket;
  string s3_access_key_id;
  string s3_secret_access_key;
public:
  S3HashStore(string bucket, string keyid, string secret_key);
  virtual string get(string key);
  virtual string put(string data);
  virtual string put_unless(string data, string key);
  virtual void ls(list<string> &l);
};

class LRUStore : public HashStore {
private:
  unsigned int limit;
  list<pair<string,string> > cache;
  HashStore *store;
public:
  LRUStore(HashStore *store) {
    this->store = store;
    limit = 200;
  }

  virtual string get(string key) {
    list<pair<string,string> >::iterator i;
    for(i = cache.begin(); i != cache.end(); i++)
      if(i->first == key) {
	string data = i->second;
	pair<string,string> p = *i;
	cache.erase(i);
	cache.push_front(p);
	return data;
      }
    
    // not in cache
    cout << "fetch " << key << "\n";
    string data = store->get(key);
    if(cache.size() >= limit)
      cache.pop_back();
    cache.push_front(pair<string,string>(key,data));
    return data;
  }

  virtual string put_unless(string data, string key) {
    return store->put_unless(data, key);
  }

  virtual string put(string data) {
    return store->put(data);
  }
  
  virtual void ls(list<string> &l) {
    store->ls(l);
  }
};

}; // end namespace

#endif
